///////////////////////////////////////////////////////////////////
//                                                               //
//   This short programs implements the RadixListSort            //
//                                                               //
//   Although theoritically efficient, it is a bit slow          //
//   'cause of all the indirections, compared to, say, heapsort  //
//   in an array.                                                //
//                                                               //
//   The biggest trick of this algorithm (if it's a trick at     //
//   all) is that since we have list, we can avoid moving the    //
//   nodes contents and only deal with their pointers. No        //
//   allocation or deallocation of any kind are required to      //
//   sort the list. Further more, basic radix sort (well, the    //
//   version I figured out) is that you distribute elements      //
//   into stacks (each being a bin or bucket). You then un-      //
//   stack the elements into the list. Eventually things get     //
//   sorted. In this algorithm, the unstacking is done by        //
//   simply stiching the lists together with some pointer        //
//   gymnastic. Result, you cut your manipulation in half.       //
//   So this implementation is about twice as fast as a          //
//   straightforward implementation of the radix sort on lists.  //
//                                                               //
//   (c) 2002 Steven Pigeon (pigeon@iro.umontreal.ca)            //
//                                                               //
///////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <time.h>     

///////////////////////////////////////////////////
//                                               //
// very basic simple-linked list definitions     //
//                                               //
///////////////////////////////////////////////////

////////////////////////
// A simple list node //
////////////////////////
class cSimpleListNode
       {
        public:

         int number; // the number; could be anything else (''body'' of node)
         cSimpleListNode *next; // linker (we do not need prev, in this application)

         cSimpleListNode(int i=0); // cheapo constructor to set next to NULL and number to i (defaults to 0)
       };

/////////////////////////
// A simple list class //
/////////////////////////
class cSimpleList
       {
        public: 

         int count; // counts the number of elements

         cSimpleListNode *head,*tail; // pointers to head and tail of list

         void add(int); // adds a number at the end of the list
         void add(cSimpleList &); // appends a LIST to this list

         void removeHead(); // removes the node pointed by the head;
         void transfer(cSimpleList &); // transfers the head of a list at the end of this list

         void unlink(); // detaches the list and this object
         void deAlloc(); // does removeHead() until (count==0)

         cSimpleList();
         ~cSimpleList();
       };


/////////////////////////////////////
//                                 //
// very basic list implementation. //
//                                 //
/////////////////////////////////////

////////////////////////////////////////////////
cSimpleListNode::cSimpleListNode(int i)
 {
  next=NULL;
  number=i;
 }

////////////////////////////////////////////////
void cSimpleList::add(int i)
 {
  cSimpleListNode *temp = new cSimpleListNode(i);

  count++;

  if (tail==NULL)
       {
        tail=head=temp; // first node!
       }
  else {
        tail->next = temp; // let's append this to the list
        tail = temp;
       }
 }

////////////////////////////////////////////////
void cSimpleList::transfer(cSimpleList & t)
 {
  if (t.count) // t is not empty?
       {
        if (head==NULL)
             {
              head=tail=t.head;
             }
        else {
              tail->next = t.head;
              tail=t.head;
             }

        t.head = t.head->next;
        if (t.head==NULL) t.tail=NULL;
        tail->next=NULL;  // to avoid linking the rest of list t

        t.count--; // removed the head, so t's count goes down by 1
        count++;
       }
 }

////////////////////////////////////////////////
void cSimpleList::add(cSimpleList & t)
 {
  if (t.count) // nothing happens if t is empty
       {
        if (tail==NULL)
             {
              head  = t.head;
              tail  = t.tail;
              count = t.count;
             }
        else {
              tail->next = t.head;
              tail = t.tail;
              count += t.count;
             }
       }
 }

////////////////////////////////////////////////
void cSimpleList::removeHead()
 {
  if (head)
       {
        cSimpleListNode *t = head->next;
        delete head;

        head = t;
        if (t==NULL) tail=NULL; // since... 

        count--;
       }
 }

////////////////////////////////////////////////
void cSimpleList::deAlloc()
 {
  while (count) removeHead();
 }

////////////////////////////////////////////////
cSimpleList::cSimpleList()
 {
  count = 0;
  head = tail = NULL;
 }

////////////////////////////////////////////////
//                                            //
// Do not destroy list, only unlinks this     //
// cSimpleList object and the list            //
//                                            //
////////////////////////////////////////////////
void cSimpleList::unlink()
 {
  count = 0;
  head = tail = NULL; 
 }

////////////////////////////////////////////////
//                                            //
// default destructor doesn't destroy the     //
// list, it simply unlinks it                 //
//                                            //
////////////////////////////////////////////////
cSimpleList::~cSimpleList()
 {
  unlink();
 }


///////////////////////////////////////////////////
//                                               //
// listRadixSort                                 //
//                                               //
// Basic Radix Sort, user determined base        //
//                                               //
///////////////////////////////////////////////////


const int LOG_BASE   = 8; // number of bits to use in each hash (arbitrary, but >= 1)
const int BASE       = 1 << LOG_BASE; // Must be a power of 2 in this implementation
const int MASK       = BASE-1; 
const int ITERATIONS = (32 + LOG_BASE -1)/LOG_BASE; // allows for logbase not divisor of 32 (==roof(32 / LOG_BASE)

void listRadixSort(cSimpleList & list)
 {
  cSimpleList buckets[BASE];

  for (int shift=0,i=0;i<ITERATIONS;i++,shift+=LOG_BASE)
   {
    // let us shuffle each element in its appropriate bin
    while (list.count>0) // while there's something left in the list
     {
      const int num = list.head->number; // to compute the hash
      const int buck = (num >> shift) & MASK;
      buckets[buck].transfer(list); // transfers the head into this bucket;
     }

    // here, list is empty... copying back the buckets in list
    for (int buck=0; buck<BASE; buck++)
     {
      list.add(buckets[buck]); // appends the buck-th bucket to the list
      buckets[buck].unlink();  // detaches the list from this bucket
     }
   } 
 }

//////////////////////////////////
void InitList(cSimpleList & t, int length)
 {
  while (length)
   {
    t.add(rand()*rand());
    length--;
   }
 }

//////////////////////////////////
void DisplayList(cSimpleList & t)
 {
  cSimpleListNode *current = t.head;

  while (current)
   {
    printf("%d\n", current->number);
    current = current->next;
   }
 }

//////////////////////////////////
void main()
 {
  clock_t start, stop;
  cSimpleList z;


  InitList(z,10000); // Fills list with random numbers

  start=clock();
  while (start==clock()); // synchronizes to clock. 
  start=clock(); 

  listRadixSort(z); // sorts list

  stop=clock();

  // display results
  printf("dt = %0.2fs\n", (stop-start)/(double)CLOCKS_PER_SEC);
  //DisplayList(z);

  z.deAlloc();
 }