require 'osx/cocoa'

class MyDocument < OSX::NSDocument

	ib_outlets		:text_view, :incremental_search_field
	
	ns_overrides	:windowNibName,
					:windowControllerDidLoadNib_,
					:loadDataRepresentation_ofType_,
					:dataRepresentationOfType_


	def windowNibName
		# return the name of the document
		"MyDocument"
	end

	#
	# File loading and saving
	#
	# If the text contains no attachments (such as images or hyperlinks),
	# we can store it as RTF. Otherwise, we need to use an ".rtfd" file
	# package, which is a directory that Mac OS X applications pretend is a file.
	# A smarter implementation would convert between the two formats on the fly,
	# as needed, notifying the user of the change as it does so.
	#
	def windowControllerDidLoadNib(controller)
		super_windowControllerDidLoadNib(controller)

		unless @loadThisData == nil

			# The following could be expressed more compactly using Ruby's "eval" operator:
			# eval "@text_view.replaceCharactersInRange_with#{@loadTypeString.to_s}	\
			#					(	0...@text_view.string.length, @loadThisData)"
			# but I prefer the extra level of errror-checking provided by the case statement.
			case @loadTypeString.to_s
				when "RTF"
					@text_view.replaceCharactersInRange_withRTF(0...@text_view.string.length, @loadThisData)
				when "RTFD"
					@text_view.replaceCharactersInRange_withRTFD(0...@text_view.string.length, @loadThisData)
				else
					# this should not happen
					print "bogus file type '#{@loadTypeString.to_s}'"
			end
	
			@loadTypeString = nil
			@loadThisData = nil
		end

		#enable automatic multiple undo
		@text_view.setAllowsUndo(1)

		# get controlTextDidChange notifications
		@incremental_search_field.setDelegate( self )
	end

	# file data accessors
	def dataRepresentationOfType(typeString)

		text_length = @text_view.string.length
		file_wrapper = @text_view.textStorage.RTFDFileWrapperFromRange(0...text_length,
													:documentAttributes,
													nil)
		case typeString.to_s
			when "RTF"
				data = @text_view.RTFFromRange(0...text_length)
			when "RTFD"
				data = @text_view.RTFDFromRange(0...text_length)
			else
				# this should not happen
				print "bogus file type '#{typeString}'"
		end

		data
	end

	def loadDataRepresentation_ofType(data, typeString)
		# the nib isn't loaded yet, so we can't directly read the the data into the view
		@loadThisData	= data
		@loadTypeString	= typeString
		1
	end
	
	#
	# Incremental search support
	#
	def controlTextDidChange(notification)
		set_incremental_search( @incremental_search_field.stringValue.to_s, true )
	end

	def set_incremental_search( searchTerm, isRegexp )

		textStorage	= @text_view.textStorage
		string		= textStorage.string
		found		= false

		begin
			# use Ruby for regexps;
			# Cocoa is faster for a plain string search, because we don't need to
			# convert the text in that case.
			if isRegexp then
				expr = Regexp.new(searchTerm)
				match = expr.match(string.to_s)
	
				if (match != nil) and (match.length > 0) then
					match_offset = match.offset(0)
					range = match_offset[0]...match_offset[1]
					found = true
				end
			else
				range = string.rangeOfString(searchTerm)
				found = true unless range.location == OSX::NSNotFound
			end

			# Note that 'range' may be either a Cocoa or a Ruby range at this point.
			# RubyCocoa's automatic data conversion will handle either case correctly.
			if found then
				@text_view.setSelectedRange(range)
				@text_view.scrollRangeToVisible(range)
			end

		rescue => local_exception
			OSX::NSBeep
			print local_exception.to_s
		end
		
	end
end
